package xyz.lychee.lagfixer.managers;

import java.util.HashMap;
import java.util.Iterator;
import org.bukkit.Bukkit;
import xyz.lychee.lagfixer.LagFixer;
import xyz.lychee.lagfixer.modules.ConsoleFilterModule;
import xyz.lychee.lagfixer.modules.CustomAiModule;
import xyz.lychee.lagfixer.modules.CustomVehiclesModule;
import xyz.lychee.lagfixer.modules.EntityLimiterModule;
import xyz.lychee.lagfixer.modules.InstantLeafDecayModule;
import xyz.lychee.lagfixer.modules.LagMonitorModule;
import xyz.lychee.lagfixer.modules.RedstoneLimiterModule;
import xyz.lychee.lagfixer.modules.WorldCleanerModule;
import xyz.lychee.lagfixer.objects.AbstractManager;
import xyz.lychee.lagfixer.objects.AbstractModule;
import xyz.lychee.lagfixer.utils.TimingUtil;

public class ModuleManager extends AbstractManager {
   private static ModuleManager instance;
   private final HashMap<Class<? extends AbstractModule>, AbstractModule> modules = new HashMap();

   public ModuleManager(LagFixer plugin) {
      super(plugin);
      instance = this;
      this.add(new CustomAiModule(plugin, this), new LagMonitorModule(plugin, this), new RedstoneLimiterModule(plugin, this), new EntityLimiterModule(plugin, this), new ConsoleFilterModule(plugin, this), new WorldCleanerModule(plugin, this), new CustomVehiclesModule(plugin, this), new InstantLeafDecayModule(plugin, this));
   }

   public <T extends AbstractModule> T get(Class<T> clazz) {
      return (AbstractModule)clazz.cast(this.modules.get(clazz));
   }

   private void add(AbstractModule... arrModules) {
      AbstractModule[] var2 = arrModules;
      int var3 = arrModules.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         AbstractModule module = var2[var4];
         this.modules.put(module.getClass(), module);
      }

   }

   public void load() {
      Iterator var1 = this.modules.values().iterator();

      while(var1.hasNext()) {
         AbstractModule module = (AbstractModule)var1.next();
         boolean enabled = this.getPlugin().getConfig().getBoolean("modules." + module.getName() + ".enabled");

         try {
            TimingUtil t = TimingUtil.startNew();
            boolean success = module.loadAllConfig();
            if (enabled) {
               module.load();
               if (success) {
                  this.getPlugin().getLogger().info(" &8• &rSucessfully loaded module " + module.getName() + " in " + t.stop().getExecutingTime() + "ms.");
               } else {
                  this.getPlugin().getLogger().info(" &8• &rSkipping unsupported module " + module.getName() + " for " + Bukkit.getServer().getBukkitVersion() + ".");
               }
            }

            module.setLoaded(success && enabled);
            Thread.yield();
         } catch (Exception var6) {
            Exception ex = var6;
            module.setLoaded(false);
            this.getPlugin().getLogger().info(" &8• &cSkipping module " + module.getName() + ", reason: " + ex.getMessage());
            this.getPlugin().printError(ex);
         }
      }

   }

   public void disable() {
      Iterator var1 = this.modules.values().iterator();

      while(var1.hasNext()) {
         AbstractModule module = (AbstractModule)var1.next();
         if (module.isLoaded()) {
            try {
               TimingUtil t = TimingUtil.startNew();
               module.disable();
               this.getPlugin().getLogger().info(" • Sucessfully disabled module " + module.getName() + " in " + t.stop().getExecutingTime() + "ms.");
               Thread.yield();
            } catch (Exception var4) {
               Exception ex = var4;
               this.getPlugin().getLogger().info(" • Error with disabling module " + module.getName() + ", reason: " + ex.getMessage());
               this.getPlugin().printError(ex);
            }
         }
      }

   }

   public boolean isEnabled() {
      return true;
   }

   public HashMap<Class<? extends AbstractModule>, AbstractModule> getModules() {
      return this.modules;
   }

   public static ModuleManager getInstance() {
      return instance;
   }
}
